<?php
namespace Teknisa\PLA\Modules\Prato\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;


class Pla13000GrupoDePratos extends \Teknisa\GER\Controllers\TeknisaCrud{

    protected $crudService;
    protected $parameterBag;
	protected $util;
	protected $connection;
    protected $dataSourceManager;
    protected $sheetImporter;
    protected $restrContrService;

	public function __construct($crudService, $parameterBag, $util, $connection, $dataSourceManager, $sheetImporter, $restrContrService) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->util = $util;
        $this->connection = $connection;
        $this->dataSourceManager = $dataSourceManager;
        $this->sheetImporter = $sheetImporter;
        $this->restrContrService = $restrContrService;
    }

    public function validateDuplicatedRecords(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->util->validateDuplicatedRecords($row, $row["dataSourceName"], $row["fieldsToValidate"]);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function saveGrupo($row, $response) {
        try {
            $dataSourceName = "grupoprato";
            $row["grupo"]["NRGRUPOPRATO"] = $this->restrContrService->getNewCode($dataSourceName);
            unset($row["grupo"]['__new_code_pk']);   
            $dataSet = new DataSet($dataSourceName, array($row["grupo"]));
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
            $nrgrupoprato = $rowsToSave[0]["NRGRUPOPRATO"];
            $dataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($dataSet);
            $response->addDataSet($dataSet);

            return $nrgrupoprato;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveServico(DTO\Request $request, DTO\Response $response) {
        try {
            $dataSourceName = "grupopratoserv";
            $row = $request->getRow();
            $this->connection->beginTransaction();
            if (isset($row["grupo"])) {
                $nrgrupoprato = $this->saveGrupo($row, $response);
            } 
            
            if (isset($row["servico"])) {
                $row["servico"]["NRGRUPOPRATO"] = isset($nrgrupoprato) ? $nrgrupoprato : $row["servico"]["NRGRUPOPRATO"];
                $dataSet = new DataSet($dataSourceName, array($row["servico"]));
                $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
                $rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
                $nrgrupopratoserv = $rowsToSave[0]["NRGRUPOPRATOSERV"];
                $dataSet = new DataSet($dataSourceName, $rowsToSave);
                $this->dataSourceManager->persist($dataSet);
                $response->addDataSet($dataSet);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function savePratos(DTO\Request $request, DTO\Response $response) {        
        try {
            $this->dataSourceName = "itgrupopratoserv";
            $row = $request->getRow();
            $this->connection->beginTransaction();
            if (isset($row["grupo"])) {
                $nrgrupoprato = $this->saveGrupo($row, $response);
            } 
            
            if (isset($row["servico"])) {
                $row["servico"]["NRGRUPOPRATO"] = isset($nrgrupoprato) ? $nrgrupoprato : $row["servico"]["NRGRUPOPRATO"];
                $nrgrupopratoserv = $this->saveServico($row["servico"], $response);
            }

            $duplicatedRows = array();        
            foreach ($row["pratos"] as $row) {
                try { 
                    $row["NRGRUPOPRATOSERV"] = isset($nrgrupopratoserv) ? $nrgrupopratoserv : $row["NRGRUPOPRATOSERV"];
                    $this->util->validateDuplicatedRecords($row, "validate_duplicated_itgrupopratoserv", array("CDFILIAL" , "CDSERVFILI", "CDPRATO", "CDFILAUXPRAT", "IDTIPORESTRICAO"));
                    $dataSet = new DataSet($this->dataSourceName, array($row));
                    $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
                    parent::save($newRequest, $response);
                } catch(\Exception $e) {
                    if ($e->getMessage() == "registerExists") {
                        array_push($duplicatedRows, $row['NMPRATO']);
                    } else {
                        throw $e; 
                    }
                }
            }

            $this->connection->commit();
            if (!empty($duplicatedRows)) {
                $response->addDataSet(new DataSet("errorDuplicated", $duplicatedRows));             
            } 
        } catch(\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
        
    }

    public function removeGrupo(DTO\Request $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $this->connection->beginTransaction();
            foreach ($rows as $row) {
                $this->deleteItgrupopratoservByNrgrupoprato($row["NRGRUPOPRATO"]);
                $this->deleteGrupopratoservByNrgrupoprato($row["NRGRUPOPRATO"]);
                $this->deleteGrupopratoByPks($row["NRGRUPOPRATO"]);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function removeServico(DTO\Request $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $this->connection->beginTransaction();
            foreach ($rows as $row) {
                $this->deleteItgrupopratoservByNrgrupopratoserv($row["NRGRUPOPRATOSERV"]);
                $this->deleteGrupopratoservByPks($row["NRGRUPOPRATOSERV"]);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function servicosExport(DTO\Request $request, DTO\Response $response) {        
        try {
            $row = $request->getRow();
            $this->connection->beginTransaction();
            $tpexport = $row["TPExport"];
            $nrgrupopratoorig = $row["NRGRUPOPRATO"];
            
            foreach ($row["servicos"] as $servico) {
                $newRow = array();
                $nrgrupopratoservorig = $servico["NRGRUPOPRATOSERV"];
                $pratos = $this->getPratos($nrgrupopratoservorig);
                
                $servico["NRGRUPOPRATOSERV"] = null;
                $servico["NRGRUPOPRATOSERVORIG"] = $nrgrupopratoservorig;
                $servico["NRGRUPOPRATO"] = $nrgrupopratoorig;
                
                $servico["__using_unit"] = false;
                $servico["__is_new"] = true;
                $servico["__new_code_pk"] = "GRUPOPRATOSERV";
                $servico["__table_pk"] = "NRGRUPOPRATOSERV";
                $servico["__new_code_length"] = 10;
                $servico["__cast_number"] = false;
                
                $newRow["servico"] = $servico;
                
                $nrgrupopratoserv = $this->saveServicoExport($newRow["servico"], $tpexport);
                foreach ($pratos as &$prato) {
                    $prato["NRGRUPOPRATOSERV"] = $nrgrupopratoserv;
                }
                $newRow["pratos"] = $pratos;
                $this->savePratosExport($newRow["pratos"]);
                $this->deleteGrupopratoservByPks($nrgrupopratoservorig);
            }
            $this->connection->commit();
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    private function getPratos($nrgrupopratoservorig) {
        $conditions = array(
            array(
                "columnName" => "NRGRUPOPRATOSERV",
                "operator" => "=",
                "value" => $nrgrupopratoservorig
            )
        );
        return $this->crudService->getFilteredDataSet("itgrupopratoserv", $conditions)->getRows();
    }

    public function savePratosExport($pratos) {
        $dataSourceName = "itgrupopratoserv";
        $dataSet = new DataSet($dataSourceName, $pratos);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
        if (!empty($rowsToSave)) {
            $dataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($dataSet);
        }
    }

    public function saveServicoExport($servico, $tpexport, $idexport = "servicos") {
        $conditions = array();
        $columns = array("NRGRUPOPRATO", "CDSERVFILI");
        foreach ($columns as $column) {
            array_push($conditions, array(
                "columnName" => $column,
                "operator"   => '=',
                "value"      => $servico[$column]
            ));
        }
        $dataSet = $this->crudService->getFilteredDataSet("grupopratoserv", $conditions);

        if (!empty($dataSet->getRows())) {
            $nrgrupopratoserv = $dataSet->getRows()[0]["NRGRUPOPRATOSERV"];
            if ($tpexport == "S") {
                if ($idexport == "servicos") {
                    $this->deleteGrupopratoservByPks($nrgrupopratoserv);
                    unset($nrgrupopratoserv);
                } else if ($idexport == "pratos") {
                    $this->deleteItgrupopratoservByNrgrupopratoserv($nrgrupopratoserv);
                }
            }
        }
        if (!isset($nrgrupopratoserv)) {
            $dataSourceName = "grupopratoserv";
            $dataSet = new DataSet($dataSourceName, array($servico));
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
            $nrgrupopratoserv = $rowsToSave[0]["NRGRUPOPRATOSERV"];
            $dataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($dataSet);
        }
        return $nrgrupopratoserv;
    }

    private function deleteGrupopratoByPks($nrgrupoprato) {
        $sql = "DELETE FROM GRUPOPRATO WHERE NRGRUPOPRATO = :NRGRUPOPRATO AND NRORG = :NRORG";
        $params = array(
            "NRGRUPOPRATO"      => $nrgrupoprato,
            "NRORG"             => $this->crudService->getEnvironment()->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }
    private function deleteGrupopratoservByPks($nrgrupopratoserv) {
        $sql = "DELETE FROM GRUPOPRATOSERV WHERE NRGRUPOPRATOSERV = :NRGRUPOPRATOSERV AND NRORG = :NRORG";
        $params = array(
            "NRGRUPOPRATOSERV"  => $nrgrupopratoserv,
            "NRORG"             => $this->crudService->getEnvironment()->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }

    private function deleteGrupopratoservByNrgrupoprato($nrgrupoprato) {
        $sql = "DELETE FROM GRUPOPRATOSERV WHERE NRGRUPOPRATO = :NRGRUPOPRATO AND NRORG = :NRORG";
        $params = array(
            "NRGRUPOPRATO"      => $nrgrupoprato,
            "NRORG"             => $this->crudService->getEnvironment()->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }

    private function deleteItgrupopratoservByNrgrupoprato($nrgrupoprato) {
        $sql = "DELETE FROM ITGRUPOPRATOSERV WHERE NRGRUPOPRATOSERV IN (SELECT NRGRUPOPRATOSERV FROM GRUPOPRATOSERV WHERE NRGRUPOPRATO = :NRGRUPOPRATO AND GRUPOPRATOSERV.NRORG = :NRORG) AND ITGRUPOPRATOSERV.NRORG = :NRORG";
        $params = array(
            "NRGRUPOPRATO"      => $nrgrupoprato,
            "NRORG"             => $this->crudService->getEnvironment()->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }

    private function deleteItgrupopratoservByNrgrupopratoserv($nrgrupopratoserv) {
        $sql = "DELETE FROM ITGRUPOPRATOSERV WHERE NRGRUPOPRATOSERV = :NRGRUPOPRATOSERV AND NRORG = :NRORG";
        $params = array(
            "NRGRUPOPRATOSERV"  => $nrgrupopratoserv,
            "NRORG"             => $this->crudService->getEnvironment()->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }

    private function deleteItgrupopratoservByPks($nritgrupopratoserv) {
        $sql = "DELETE FROM ITGRUPOPRATOSERV WHERE (INSTR(:NRITGRUPOPRATOSERV, '_' || NRITGRUPOPRATOSERV || '_') <> 0) AND NRORG = :NRORG";
        $params = array(
            "NRITGRUPOPRATOSERV"  => $nritgrupopratoserv,
            "NRORG"               => $this->crudService->getEnvironment()->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }

    public function pratosExport(DTO\Request $request, DTO\Response $response) {        
        try {
            $row = $request->getRow();
            $this->connection->beginTransaction();
            $cdservfiliOrig = $row["CDSERVFILIORIG"];
            
            $servicosDest = $row["NMSERVICO"];
            $exportServOrig = false;
            $newServicosDest = array();
            foreach ($servicosDest as $servicoDest) {
                if ($servicoDest == $cdservfiliOrig) {
                    $exportServOrig = true;
                } 
                else {
                    array_push($newServicosDest, $servicoDest);
                }
            }
            $pratos = $row["pratos"];
            $duplicatedRows = array();
            $rowsToSave = array();
            foreach ($pratos as $prato) {
                $servicosDest = $row["NMSERVICO"];
                $newPratos = array();
                $servicosDuplicated = array();
                $servicos = $this->getItgrupopratoserv($row["CDFILIAL"], $prato["CDPRATO"], $prato["CDFILAUXPRAT"], implode("_", $newServicosDest), $row["IDTIPORESTRICAO"]);
                foreach($servicos as $servico) {
                    array_push($duplicatedRows, array(
                        "CDPRATO" => $prato["CDPRATO"],
                        "NMPRATO" => $prato["NMPRATO"],
                        "CDFILIAL" =>  $row["CDFILIAL"],
                        "CDGRUPOPRATO" =>  $servico["CDGRUPOPRATO"],
                        "NMGRUPOPRATO" =>  $servico["NMGRUPOPRATO"],
                        "CDSERVFILI" =>  $servico["CDSERVFILI"],
                        "NMSERVICO" =>  $servico["NMSERVICO"],
                        "TIPO" => "DUPLICADO"
                    ));
                    array_push($servicosDuplicated, $servico["CDSERVFILI"]);
                }
                $pratosAtivos = $this->getPratoAtivo($row["CDFILIAL"], $prato["CDPRATO"], $prato["CDFILAUXPRAT"], implode("_", $newServicosDest));
                foreach($pratosAtivos as $pratoAtivo) {
                    if($pratoAtivo["PRATOATIVO"] == "N"){
                        array_push($duplicatedRows, array(
                            "CDPRATO" => $prato["CDPRATO"],
                            "NMPRATO" => $prato["NMPRATO"],
                            "CDFILIAL" =>  $row["CDFILIAL"],
                            "CDGRUPOPRATO" =>  "",
                            "NMGRUPOPRATO" =>  "",
                            "CDSERVFILI" =>  $pratoAtivo["CDSERVFILI"],
                            "NMSERVICO" =>  $pratoAtivo["NMSERVICO"],
                            "TIPO" => "INATIVO"
                        ));
                        array_push($servicosDuplicated, $pratoAtivo["CDSERVFILI"]);
                    }
                }
                $servicosDest = array_diff($servicosDest, $servicosDuplicated);
                
                $newNrgrupopratoserv = array();
                foreach ($servicosDest as $servico) {
                    $newRowServico = array();
                    $newRowServico["__using_unit"] = false;
                    $newRowServico["__is_new"] = true;
                    $newRowServico["__new_code_pk"] = "GRUPOPRATOSERV";
                    $newRowServico["__table_pk"] = "NRGRUPOPRATOSERV";
                    $newRowServico["__new_code_length"] = 10;
                    $newRowServico["__cast_number"] = false;
                    $newRowServico["NRGRUPOPRATO"] = $row["NRGRUPOPRATO"];
                    $newRowServico["CDSERVFILI"] = $servico;
                    $newRowServico["NRAGRUPGRPRAT"] = $row["NRAGRUPGRPRAT"];
                    array_push($newNrgrupopratoserv, $this->saveServicoExport($newRowServico, $row["TPExport"], "pratos"));
                }
                foreach($newNrgrupopratoserv as $nrgrupopratoserv) {

                    $prato["NRITGRUPOPRATOSERV"] = null;
                    $prato["NRGRUPOPRATOSERV"] = $nrgrupopratoserv;
                    $prato["__using_unit"] = false;
                    $prato["__is_new"] = true;
                    $prato["__new_code_pk"] = "ITGRUPOPRATOSERV";
                    $prato["__table_pk"] = "NRITGRUPOPRATOSERV";
                    $prato["__new_code_length"] = 10;
                    $prato["__cast_number"] = false;
                    array_push($rowsToSave, $prato);
                }
            }
            $dataSourceName = "itgrupopratoserv";
            $newDataSet = new DataSet($dataSourceName, $rowsToSave);
            $newRequest = new DTO\Request\DataSet($newDataSet, "", "", "");
            $rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
            $newDataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($newDataSet);
            if ($row["deletePratosOrig"] == true || $exportServOrig) {
                $nritgrupopratoservToDelete = array();
                foreach ($pratos as $prato) {
                    array_push($nritgrupopratoservToDelete, $prato["NRITGRUPOPRATOSERV"]);
                }
                $nritgrupopratoservToDelete = implode('_', $nritgrupopratoservToDelete);
                $nritgrupopratoservToDelete = '_'.$nritgrupopratoservToDelete.'_';
                $this->deleteItgrupopratoservByPks($nritgrupopratoservToDelete); 
            }

            $this->connection->commit();
            if (!empty($duplicatedRows)) {
                $response->addDataSet(new DataSet("errorDuplicated", $duplicatedRows));             
            } 
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    private function getItgrupopratoserv($cdfilial, $cdprato, $cdfilauxprat, $cdservfili, $idtiporestricao) {
        $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $cdfilial
            ),
            array(
                "columnName" => "CDFILAUXPRAT",
                "operator" => "=",
                "value" => $cdfilauxprat
            ),
            array(
                "columnName" => "CDPRATO",
                "operator" => "=",
                "value" => $cdprato
            ),
            array(
                "columnName" => "CDSERVFILI",
                "operator" => "=",
                "value" => $cdservfili
            ),
            array(
                "columnName" => "IDTIPORESTRICAO",
                "operator" => "=",
                "value" => $idtiporestricao
            )
        );
        return $this->crudService->getFilteredDataSet("pla13000_itgrupopratoserv_servicos", $conditions)->getRows();
    }
    
    private function getPratoAtivo($cdfilial, $cdprato, $cdfilauxprat, $cdservfili) {
        $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $cdfilial
            ),
            array(
                "columnName" => "CDFILAUXPRAT",
                "operator" => "=",
                "value" => $cdfilauxprat
            ),
            array(
                "columnName" => "CDPRATO",
                "operator" => "=",
                "value" => $cdprato
            ),
            array(
                "columnName" => "CDSERVFILI",
                "operator" => "=",
                "value" => $cdservfili
            )
        );
        return $this->crudService->getFilteredDataSet("pla13000_pratoativo_filiserv", $conditions)->getRows();
    }    

    public function importSheet(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();

            $sheetPath = $this->sheetImporter->createTemporaryFile($row["sheetImport"]);
            $worksheet = $this->sheetImporter->loadSheet($sheetPath);
            $sheet = $this->sheetImporter->parseSheet($worksheet);

            $this->sheetImporter->validateSheetColumns($sheet);
            $this->sheetImporter->validateSheetRows($sheet, $response);

            $nrimpgrupoprato = $this->sheetImporter->saveSheetData($sheet, $response, $row);
            $nrlogimpgrupoprato = $this->sheetImporter->executeProcedure($nrimpgrupoprato, $response);

        } catch(\Exception $e) {

            $response->setError(new Error($e->getMessage(), 500));

        } finally {

            if (isset($sheetPath)) {
                $this->sheetImporter->removeTemporaryFile($sheetPath);
            }

            if (isset($nrimpgrupoprato)) {
                $this->sheetImporter->removeSheetTmpData($nrimpgrupoprato);
            }

            $nrlogimpgrupoprato = $this->sheetImporter->getNrlogimpgrupoprato();
            if ($nrlogimpgrupoprato) {
                $this->sheetImporter->removeSheetLog($nrlogimpgrupoprato);
            }
        }
    }

    
} 

